#include <glib.h>

#include "rcbutton.h"
#include "interface.h"
#include "keyvalues.h"
#include "remote-udp.h"

#ifdef USE_HILDON
#include <hildon-widgets/hildon-program.h>
#include <hildon-widgets/hildon-controlbar.h>
#endif 


#define BOX_SPACING 4
#define PROGRAM_NAME "LAN Remote Control"

#define DEFAULT_PAGES 3

#ifdef HAVE_NTABLET
# define DEFAULT_WIN_WIDTH 800
# define DEFAULT_WIN_HEIGHT 480
# define DEFAULT_BUTTON_WIDTH 80
# define DEFAULT_BUTTON_HEIGHT 110
#define DEFAULT_BUTTONS_PER_LINE 9
#define DEFAULT_BUTTONS_PER_ROW 4
#else
# define DEFAULT_WIN_WIDTH 480
# define DEFAULT_WIN_HEIGHT 800
# define DEFAULT_BUTTON_WIDTH 110
# define DEFAULT_BUTTON_HEIGHT 80
#define DEFAULT_BUTTONS_PER_LINE 4
#define DEFAULT_BUTTONS_PER_ROW 9
#endif


GList *pages = NULL;

static gboolean scroll (gint pages);

static gboolean scroll_one_plus(gpointer data);

static void send_code (guint code);
static void button_pressed (RcButton *button, gpointer data);
static void button_released (RcButton *button, gpointer data);

static gboolean add_to_menu (gchar *pagename);
static gboolean create_buttons_to_page (GList *node);

static gboolean on_button_timeout(RcButton *button);

#ifdef HAVE_NTABLET
static void 
on_key_press_event (GtkWidget * localtree, GdkEventKey *event, gpointer data);
#endif


static guint button_timeout_id = 0;

static GtkObject *adj = NULL;
static GtkWidget *box = NULL;

static RemoteUdp *udp = NULL;
static RemoteNode *pressed_data = NULL;

static gint curpage = -1;

void 
set_remote_udp (RemoteUdp *remoteudp)
{
  udp = remoteudp;
}

GtkWidget *
create_main_ui (void)
{
  GtkWidget *window = NULL; 
  GtkWidget *scrolledwin = NULL;

#ifdef USE_HILDON
  HildonProgram *program;

  program = HILDON_PROGRAM (hildon_program_new());
  window = GTK_WIDGET (hildon_window_new ());
  hildon_program_add_window (program, HILDON_WINDOW(window));
  gtk_window_fullscreen(GTK_WINDOW (window));

#else	
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), PROGRAM_NAME);
  gtk_widget_set_size_request (window, DEFAULT_WIN_WIDTH, DEFAULT_WIN_HEIGHT);
#endif

#ifdef HAVE_NTABLET
  /* GDK_F6 quit */
  gtk_widget_add_events(window, GDK_KEY_PRESS_MASK);
  g_signal_connect ((gpointer) window, "key-press-event",
                    G_CALLBACK (on_key_press_event),
		    NULL);
#endif


  /* Scrolledwindow */
  adj = gtk_adjustment_new( 0.1, 0.0, 1.0, 0.001, 0.01, 1.0);

#ifdef HAVE_NTABLET
  scrolledwin = gtk_scrolled_window_new (GTK_ADJUSTMENT (adj), NULL);
#else
  scrolledwin = gtk_scrolled_window_new (NULL, GTK_ADJUSTMENT (adj));
#endif
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwin), 
				 GTK_POLICY_NEVER, 
				 GTK_POLICY_NEVER);
  gtk_widget_show (scrolledwin);

  /* Create fixed */
#ifdef USE_HILDON
  box = gtk_hbox_new(TRUE, FALSE);
#else
  box = gtk_vbox_new(TRUE, FALSE);
#endif
  gtk_widget_show (box);

  add_to_menu("programs");

  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwin), 
					 box);
  gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (scrolledwin));

  
  g_signal_connect ((gpointer) window, "destroy",
                    G_CALLBACK (gtk_main_quit),
		    NULL);

  return window;
}

/* 
  Convert guint to char (not real char) 
 */
static void 
send_code (guint code)
{
  guchar cmd[2] = { 0x00, 0x00 };

  //  g_assert (code > G_UINT_MAX);

  cmd[0] = (code & 0xff00)>>8;
  cmd[1] = (code & 0xff);
  remote_udp_send_data (udp, (gpointer)cmd);
}

static gboolean
add_to_menu (gchar *pagename)
{
  /* Create menubuttons */
  GList *page = NULL;
  GList *node = NULL;
  GList *buttondata = NULL;

  buttondata = get_buttons_to_group (pagename);
  if (buttondata == NULL) {
    g_debug ("programs/buttondata are NULL");
    return FALSE;
  }
  node = g_list_first (buttondata);

  curpage++;
  create_buttons_to_page (node);
  page = g_list_last(pages);
  if(page->data != NULL) {
    GtkWidget *table = GTK_WIDGET (page->data);
    gtk_container_add (GTK_CONTAINER (box), table);
    gtk_widget_realize(table);
  }
  return TRUE;
}

static void
back_button_pressed (GtkButton *button, gpointer data)
{
  GList *last = NULL;

  g_debug ("%s", __FUNCTION__ );

  if (curpage == 0)
    return;

  scroll (-1);
  curpage--;

  last = g_list_last (pages);
  if(last->data != NULL) {
    GtkWidget *table = GTK_WIDGET (last->data);
    gtk_container_remove (GTK_CONTAINER (box), table);
    //gtk_widget_unref(table);
  }
  pages = g_list_remove_link(pages, last);

}


static void
button_pressed (RcButton *button, gpointer data)
{
  g_assert (button != NULL);

  g_debug ("%s\n", __FUNCTION__ );
  g_debug ("Button label: %s\n", gtk_button_get_label (GTK_BUTTON (button)));
  g_object_get(button, "data", &pressed_data, NULL);

  if(pressed_data == NULL) {
    return;
  }

  g_print ("Data name: %s, key: %s and code: %d \n", 
	   pressed_data->name, pressed_data->key, pressed_data->code );

  /* Send UDP message */
  send_code ((guint)pressed_data->code);

#if 0
  /* Get GList with key if != NULL then scroll else start timeout_idle */
  buttondata = get_buttons_to_group (pressed_data->key);
  if (buttondata == NULL) {
    g_print ("buttondata is NULL");
    return;
  }
#endif

  if (add_to_menu (pressed_data->key) == FALSE) {
    if (button_timeout_id > 0) {
      if (g_source_remove (button_timeout_id) == TRUE) {
        button_timeout_id = 0;
      }
    }
    button_timeout_id = g_timeout_add (500, (GSourceFunc)on_button_timeout,
				       button);
    return;
  }

  g_idle_add(scroll_one_plus, NULL);

}

static void
button_released (RcButton *button, gpointer data)
{
  g_assert (button != NULL);

  if (button_timeout_id > 0) {
    if (g_source_remove (button_timeout_id) == TRUE) {
      button_timeout_id = 0;
    }
  }

}

static gboolean
scroll_one_plus(gpointer data)
{
  scroll(1);
  return FALSE;
}

#ifdef HAVE_NTABLET
static void
button_set_angle (GtkWidget *button, gdouble angle)
{
    GList *list = gtk_container_get_children( GTK_CONTAINER (button) );
    GtkLabel *label = list->data;
    gtk_label_set_angle (GTK_LABEL (label), angle);
}
#endif

static gboolean 
create_buttons_to_page (GList *node)
{
  gint x, y = 0;

  GtkWidget *button = NULL;
  GtkWidget *table = NULL;

#ifdef HAVE_NTABLET
  table = gtk_table_new (9, 4, TRUE);
#else
  table = gtk_table_new (4, 9, TRUE);
#endif
  gtk_widget_show (table);
#ifdef HAVE_NTABLET
  gtk_widget_set_size_request (table, 
	 		       DEFAULT_WIN_WIDTH, 
			       DEFAULT_WIN_HEIGHT*2.25);
#else
  gtk_widget_set_size_request (table, 
	 		       DEFAULT_WIN_WIDTH*2.25, 
			       DEFAULT_WIN_HEIGHT);
#endif

  /* Navigation buttons */
  button = gtk_button_new_with_label("< Back");
  if (curpage == 0) {
    gtk_widget_set_sensitive (button, FALSE);
  }
  if( button == NULL) {
    g_error ("could not create buttons");
    return FALSE;
  }
  gtk_signal_connect (GTK_OBJECT (button), "pressed",
                      GTK_SIGNAL_FUNC (back_button_pressed), NULL);
  gtk_widget_show (button);
#ifdef HAVE_NTABLET
  button_set_angle (button, 90.0 );
  gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 3, 4);
#else
  gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
#endif
  button = gtk_button_new_with_label(" ");
  if( button == NULL) {
    g_error ("could not create buttons");
    return FALSE;
  }
  gtk_widget_show (button);
#ifdef HAVE_NTABLET
  button_set_angle (button, 90.0 );
  gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 2, 3);
#else
  gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 1);
#endif

  button = gtk_button_new_with_label(" ");
  if( button == NULL) {
    g_error ("could not create buttons");
    return FALSE;
  }
  gtk_widget_show (button);
#ifdef HAVE_NTABLET
  button_set_angle (button, 90.0 );
  gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 1, 2);
#else
  gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
#endif

  button = gtk_button_new_with_label("Prefs");
  if( button == NULL) {
    g_error ("could not create buttons");
    return FALSE;
  }
  gtk_widget_show (button);
#ifdef HAVE_NTABLET
  button_set_angle (button, 90.0 );
  gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
#else
  gtk_table_attach_defaults (GTK_TABLE (table), button, 3, 4, 0, 1);
#endif


#ifdef HAVE_NTABLET
  for (x=1; x < 9; x++) {
    for (y=3; y >= 0; y--) {
#else
  for (y=1; y < 9; y++) {
    for (x=0; x < 4; x++) {
#endif
    RemoteNode *data = NULL;

    if (node != NULL && node->data != NULL) {
      data = (RemoteNode *)(node->data);
    }

    /* Create button and set needed data */
    button = rc_button_new_with_data ((gpointer)data);

    if( button == NULL) {
      g_error ("could not create buttons");
      return FALSE;
    }

    if (data != NULL && data->name != NULL) {
      gtk_button_set_label ( GTK_BUTTON (button), data->name);
#ifdef HAVE_NTABLET
      button_set_angle (button, 90.0 );
#endif
    }

    gtk_widget_show (button);

    gtk_signal_connect (GTK_OBJECT (button), "pressed",
                        GTK_SIGNAL_FUNC (button_pressed), NULL);
    gtk_signal_connect (GTK_OBJECT (button), "released",
                        GTK_SIGNAL_FUNC (button_released), NULL);

    gtk_table_attach_defaults (GTK_TABLE (table), button, x, x+1, y, y+1);
    node = g_list_next (node);
    }
  }
  pages = g_list_append (pages, (gpointer)table);

  return FALSE;

}


/* FIXME: yuck */
static gboolean
scroll (gint pages)
{
  gdouble scroll_value = 0.1 * curpage;
  //  gdouble scroll_value = 0.1 * curpage;
  gdouble value = 0.0;
  gdouble upper = 0.0;

  g_object_get (G_OBJECT(adj), "upper", &upper, NULL);

#ifdef HAVE_NTABLET
  if (pages < 0) {
    for( value = (gdouble)curpage; value >= (gdouble)(curpage + pages); value -= scroll_value) {
      gtk_adjustment_set_value(GTK_ADJUSTMENT (adj), (value * DEFAULT_WIN_WIDTH) ); 
    }			
  } else {
    for( value = (gdouble)curpage-1; value <= (gdouble)(curpage-1 + pages); value += scroll_value) {
      if( value * DEFAULT_WIN_WIDTH > upper )
	gtk_adjustment_set_value(GTK_ADJUSTMENT ( adj), upper);
      else
	gtk_adjustment_set_value(GTK_ADJUSTMENT ( adj), (value * DEFAULT_WIN_WIDTH));
    }
    //gtk_adjustment_set_value(GTK_ADJUSTMENT ( adj), upper);
      //    gtk_adjustment_set_value(GTK_ADJUSTMENT ( adj), (2.0 * DEFAULT_WIN_WIDTH));
  }
#else
  if (pages < 0) {
    for( value = (gdouble)curpage; value >= (gdouble)(curpage + pages); value -= scroll_value) {
      gtk_adjustment_set_value(GTK_ADJUSTMENT (adj), (value * DEFAULT_WIN_HEIGHT) ); 
    }			
  } else {
    for( value = (gdouble)curpage-1; value <= (gdouble)(curpage-1 + pages); value += scroll_value) {
      gtk_adjustment_set_value(GTK_ADJUSTMENT ( adj), (value * DEFAULT_WIN_HEIGHT));
    }
    gtk_adjustment_set_value(GTK_ADJUSTMENT ( adj), (2.0 * DEFAULT_WIN_HEIGHT));
  }
#endif

  return FALSE;

}

static gboolean
on_button_timeout(RcButton *button)
{
  g_debug (__FUNCTION__);
  g_debug ("pressed data: %u", (guint)pressed_data->code);
  /* Send UDP message */
  send_code ((guint)pressed_data->code);

  return TRUE;
}

#ifdef HAVE_NTABLET
static void
on_key_press_event (GtkWidget *window, GdkEventKey *keyevent, gpointer data)
{
  if (keyevent->keyval == GDK_F6) {
    gtk_main_quit ();
  }
}
#endif

