#include <stdlib.h>
#include <glib.h>

#include  "keyvalues.h"

static void free_all_buttons (void);
static RemoteNode *find_node_from_buttons (gchar *key);
static GList *init_all_buttons (void);

static void read_config (void);


static GKeyFile *keyfile = NULL;
static guint16 code = 1; /* lets stay away from 0 */

static GList *all_buttons = NULL;
static gchar *host = NULL;
static guint port = 0;

guint 
get_port (void)
{
  return port;
}

gchar *
get_host (void)
{
  return host;
}

GList *
get_buttons_to_group (char *group)
{
  RemoteNode *node = NULL;
  GError *error = NULL;
  GList *list = NULL;

  gchar **keys = NULL;
  gsize numkeys = 0;
  guint i = 0;

  if ( g_key_file_has_group (keyfile, group) == FALSE)
    return NULL;

  keys = g_key_file_get_keys (keyfile, group, &numkeys, &error);
  if (error != NULL) {
    g_error ("Error: %s", error->message);
    g_clear_error (&error);
    return NULL;
  }

  for (i=0; i<numkeys; i++) {
    g_print ("keys[%d]: = '%s'\n", i, keys[i]);
    node = find_node_from_buttons (keys[i]);
    if (node != NULL) {
      g_print ("node->name = '%s'\n",node->name);
      list = g_list_append (list, (gpointer)node );
    }
  }
  g_strfreev (keys);

  return list;
}


static RemoteNode *
find_node_from_buttons (gchar *key)
{
  RemoteNode *data = NULL;
  GList *node = NULL;

  g_print ("%s\n", __FUNCTION__);

  node = g_list_first (all_buttons);
  while (node) {
    data = (RemoteNode *)node->data;
    if (g_strrstr (key, data->key) != NULL) {
      g_print ("returning: %s\n",data->key);
      return data;
    }
    node = g_list_next (node);
  }

  return NULL;
}


static GList *
init_all_buttons (void)
{
  RemoteNode *node = NULL;
  GError *error = NULL;
  GList *list = NULL;
  gchar **keys = NULL;
  gchar *value = NULL;
  gsize numkeys = 0;
  guint i = 0;

  keys = g_key_file_get_keys (keyfile, "buttons", &numkeys, &error);
  if (error != NULL) {
    g_error ("Error: %s", error->message);
    g_clear_error (&error);
    return NULL;
  }

  for (i=0; i<numkeys; i++) {
    value = g_key_file_get_value (keyfile, "buttons", keys[i], &error);
    if (error != NULL) {
      g_error ("Error: %s", error->message);
      g_clear_error (&error);
      g_strfreev (keys);
      return NULL;
    }

    node = g_malloc (sizeof (RemoteNode));
    node->code = code++;
    if (keys[i] != NULL)
      node->key = g_strdup (keys[i]);
    if (value != NULL)
      node->name = g_strdup (value);
    list = g_list_append (list, (gpointer)node );
    g_free (value);
  }
  g_strfreev (keys);

  return list;
}

static void
read_config (void)
{
  GError *error = NULL;
  gboolean has_host = TRUE;
  gboolean has_port = TRUE;
  gchar *tmpstring = NULL; 

  if( g_key_file_has_group (keyfile, "configure") == FALSE )
    return;

  if ( g_key_file_has_key (keyfile, "configure", "host", &error) == FALSE) {
    has_host = FALSE;
  }
  if (error != NULL) {
    g_error ("Error: %s", error->message);
    g_clear_error (&error);
    error = NULL;
  }

  if (  g_key_file_has_key ( keyfile, "configure", "port", &error) == FALSE ) {
    has_port = FALSE;
  }
  if (error != NULL) {
    g_error ("Error: %s", error->message);
    g_clear_error (&error);
    error = NULL;
  }

  if ( has_host == TRUE ) {
    tmpstring = g_key_file_get_value (keyfile, "configure", "host", &error);
    if (tmpstring != NULL) {
      host = g_strdup (tmpstring);
      g_free (tmpstring);
    }
  }
  if ( has_port == TRUE ) {
    tmpstring = g_key_file_get_value(keyfile, "configure", "port", &error);
    if (tmpstring != NULL) {
      port = atoi (tmpstring);
      g_free (tmpstring);
    }
  }
  g_print ("host='%s' port='%d'\n", host, port);

  if (port == 0)
    port = 8765;
  if (host == NULL)
    host = g_strdup ("127.0.0.1");
}

 
gint
read_keyfile (const gchar *file)
{
  GError *error = NULL;
  GKeyFileFlags flags = G_KEY_FILE_KEEP_TRANSLATIONS; /* is this ok? */
  gboolean retval = FALSE;

  g_assert (keyfile == NULL);
  keyfile = g_key_file_new (); 
  if (keyfile == NULL) {
    g_error ("Could not create new keyfile\n");
    return 1;
  }
 
  retval = g_key_file_load_from_file (keyfile, file, flags, &error);
  if (error != NULL) {
    g_error ("Error: %s", error->message);
    g_clear_error (&error);
    return 1;
  }

  read_config ();

  all_buttons = init_all_buttons ();
  if (all_buttons == NULL) {
    g_error ("Could not init all buttons\n");
    return 1;
  }

  return 0;
}



void
free_keyfile (void)
{
  if (keyfile != NULL)
    g_key_file_free (keyfile);

  g_free (host);

  //  g_print ("memleak!!! - free all_buttons list and nodes here");
  if (all_buttons != NULL)
    free_all_buttons ();

}


static void
free_all_buttons (void)
{
  RemoteNode *data = NULL;
  GList *node = NULL;

  g_print ("%s\n", __FUNCTION__);

  node = g_list_first (all_buttons);
  while (node) {
    data = (RemoteNode *)node->data;
    if (data->key != NULL)
      g_free (data->key);
    if (data->name != NULL)
      g_free (data->name);
    node = g_list_next (node);
  }
  
  g_list_free (all_buttons);
  all_buttons = NULL;
}
