/*
 * * Copyright (C) 2007 Joni Valtanen <jvaltane@kapsi.fi>
 *
 * This program is free software; you can redistribute it and/or
 * modify
 * it under the terms of the GNU General Public License as published
 * by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 */

/* Network stuff */
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

#include <string.h> /* strlen */
#include <strings.h> /* bzero */

/* gobject stuff */
#include <glib-object.h>
#include <glib.h>
#include <glib/gtypes.h>

#include "remote-udp.h"

#define LIRC_DEFAULT_PORT 8765;

static gint create_udp_socket (RemoteUdp *self);
static gint send_data (RemoteUdp *self, gpointer data);


enum {
  REMOTE_UDP_PORT = 1,
  REMOTE_UDP_HOST
};


/* gobject stuff */
static GObjectClass *parent_class = NULL;

static GObject *
remote_udp_constructor (GType type, guint numb_cproperties,
		         GObjectConstructParam *cproperties)
{
  GObject *obj = NULL;
  RemoteUdpClass *klass = NULL;
  GObjectClass *parent_class = NULL;

  klass = REMOTE_UDP_CLASS (g_type_class_peek (REMOTE_TYPE_UDP));
  parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));

  obj = parent_class->constructor (type, numb_cproperties, cproperties);

  return obj;
}

static void
remote_udp_finalize (GObject *obj)
{
  RemoteUdp *self = (RemoteUdp *)obj;

  /* TODO - free udp stuff if needed */
  if (self->host != NULL)
    g_free (self->host);

}

static void
remote_udp_instance_init (GTypeInstance *inst, gpointer gclass)
{
    RemoteUdp *self = REMOTE_UDP (inst);

    self->port = 0;
    self->host = NULL;

    /* What to do with these */
    self->udp_socket = 0;

}

static void
remote_udp_set_property (GObject *object, guint pid, const GValue *value, 
			GParamSpec *pspec)
{
  RemoteUdp *self = REMOTE_UDP (object);

  switch (pid) {
    case REMOTE_UDP_PORT : {
      self->port = g_value_get_uint (value);
      break;
    }
    case REMOTE_UDP_HOST : {
      if (self->host != NULL)
        g_free ( self->host );

      self->host = g_value_dup_string (value);
      break;
    }
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(object,pid,pspec);
      break;
  }
  
}

static void
remote_udp_get_property (GObject *object, guint pid, GValue *value, 
			 GParamSpec *pspec)
{
  RemoteUdp *self = (RemoteUdp *) object;

  switch (pid) {
    case REMOTE_UDP_PORT : {
      g_value_set_uint (value, self->port);
      break;
    }
    case REMOTE_UDP_HOST : {
      g_value_set_string (value, self->host);
      break;
    }
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(object,pid,pspec);
      break;
  }
}

static void
remote_udp_class_init (gpointer g_class, gpointer g_class_data)
{
  GParamSpec *pspec = NULL;
  GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
  RemoteUdpClass *klass = REMOTE_UDP_CLASS (g_class);

  gobject_class->set_property = remote_udp_set_property;
  gobject_class->get_property = remote_udp_get_property;
  gobject_class->constructor = remote_udp_constructor;
  gobject_class->finalize = remote_udp_finalize;

  parent_class = g_type_class_peek_parent (klass);

  /* properties */
  /* port */
  pspec = g_param_spec_uint ("port",
                             "UDP port",
                             "Set/Get Network port",
                             1  /* minimum value */,
                             65535 /* maximum value */,
                             8765  /* default value */,
                             G_PARAM_READWRITE );
  g_object_class_install_property (gobject_class,
                                   REMOTE_UDP_PORT,
                                   pspec);
  pspec = g_param_spec_string ("host",
			       "Host to send packets",
			       "Set/Get host where to send packets",
			       "127.0.0.1",
			       G_PARAM_READWRITE);
  g_object_class_install_property (gobject_class,
                                   REMOTE_UDP_HOST,
                                   pspec);

}
/* global methods */

GType  
remote_udp_get_type (void)
{
  static GType type = 0;
  if (type == 0) {
    static const GTypeInfo info = {
      sizeof (RemoteUdpClass),
      NULL,   /* base_init */
      NULL,   /* base_finalize */
      remote_udp_class_init,   /* class_init */
      NULL,   /* class_finalize */
      NULL,   /* class_data */
      sizeof (RemoteUdp),
      0,      /* n_preallocs */
      remote_udp_instance_init    /* instance_init */
    };
    type = g_type_register_static (G_TYPE_OBJECT,
                                   "RemoteUdp",
                                   &info, 0);
  }
  return type;
}

RemoteUdp *
remote_udp_new (void)
{
  RemoteUdp *self = g_object_new (REMOTE_TYPE_UDP, NULL);
  return self;

}

gint 
remote_udp_send_data (RemoteUdp *self, gpointer data)
{
  gint retval = 0;

  if (self->udp_socket == 0) {
    self->udp_socket = create_udp_socket (self);
  }

  if (self->udp_socket == 0) {
    return -1;
  }

  if (self->host == NULL) {
    self->host = g_strdup ("127.0.0.1");
  }

  if (self->port == 0) {
    self->port = LIRC_DEFAULT_PORT;
  }

  /* create socket */
  if (create_udp_socket (self) == 0) {
    g_critical ("Could not create socket");
    return -1;
  }

  /* send data */
  retval = send_data (self, data);
#if 0
  if (retval != sizeof (data)) {
    g_print ("sended data (%s) does not sended\n", (gchar * )data );
  }
#endif
  /* close socket */
  close (self->udp_socket);

  return 0;
}


/* internal use only */

static gint 
create_udp_socket (RemoteUdp *self)
{
  gint retval = 0;
  int sock = 0;

  bzero (&self->addr, sizeof (self->addr));

  retval = inet_aton (self->host, &self->addr.sin_addr);
  if (retval == 0)
    return -1;

  self->addr.sin_family = AF_INET;
  self->addr.sin_port = htons (self->port);

  sock = socket (AF_INET, SOCK_DGRAM, 0);

  return sock;
}

static gint
send_data (RemoteUdp *self, gpointer data)
{
  gint retval = 0;
  char *buf = (char *)data;  
  if (buf == NULL)
    return -1;

  /* we send two marks */
  retval = (gint) sendto (self->udp_socket, buf, 2, 0, 
			  (struct sockaddr *) &self->addr, sizeof (self->addr));

  return retval;

}
